/**********************************************************************************************************************
 * \file Cpu0_Main.c
 * \copyright Copyright (C) Infineon Technologies AG 2019
 * 
 * Use of this file is subject to the terms of use agreed between (i) you or the company in which ordinary course of 
 * business you are acting and (ii) Infineon Technologies AG or its licensees. If and as long as no such terms of use
 * are agreed, use of this file is subject to following:
 * 
 * Boost Software License - Version 1.0 - August 17th, 2003
 * 
 * Permission is hereby granted, free of charge, to any person or organization obtaining a copy of the software and 
 * accompanying documentation covered by this license (the "Software") to use, reproduce, display, distribute, execute,
 * and transmit the Software, and to prepare derivative works of the Software, and to permit third-parties to whom the
 * Software is furnished to do so, all subject to the following:
 * 
 * The copyright notices in the Software and this entire statement, including the above license grant, this restriction
 * and the following disclaimer, must be included in all copies of the Software, in whole or in part, and all 
 * derivative works of the Software, unless such copies or derivative works are solely in the form of 
 * machine-executable object code generated by a source language processor.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE 
 * COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN 
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 *********************************************************************************************************************/
#include "Ifx_Types.h"
#include "IfxCpu.h"
#include "IfxScuWdt.h"

#include "Ifx_reg.h"
#include "AudioTask.h"
#include "UartTask.h"
#include "FreeRTOSConfig.h"
#include "FreeRTOS.h"
#include "task.h"

#include "lwip/api.h"
#include "lwip/tcpip.h"
#include "lwip/memp.h"
#include "Ifx_Lwip.h"


IFX_ALIGN(4) IfxCpu_syncEvent g_cpuSyncEvent = 0;

eth_addr_t ethAddr = {{0x02, 0x11, 0x22, 0x33, 0x44, 0x55}};

static __attribute__((__noreturn__)) void OsTask_IdleLed(void *arg)
{
    (void)arg;

    //Init LED Pin as output
    P00_IOCR4.B.PC6 =0x10;

    while (1)
    {
        vTaskDelay(pdMS_TO_TICKS(100));
        P00_OUT.B.P6=1;
        vTaskDelay(pdMS_TO_TICKS(1000));
        P00_OUT.B.P6=0;
    }
}

void core0_main(void)
{
    IfxCpu_enableInterrupts();
    
    /* !!WATCHDOG0 AND SAFETY WATCHDOG ARE DISABLED HERE!!
     * Enable the watchdogs and service them periodically if it is required
     */
    IfxScuWdt_disableCpuWatchdog(IfxScuWdt_getCpuWatchdogPassword());
    IfxScuWdt_disableSafetyWatchdog(IfxScuWdt_getSafetyWatchdogPassword());
    
    /* Wait for CPU sync event */
    IfxCpu_emitEvent(&g_cpuSyncEvent);
    IfxCpu_waitEvent(&g_cpuSyncEvent, 1);
    
    /*Initialize the LwIp. Set the MAC Address and IP Address 192.168.2.218*/
    tcpip_init( (tcpip_init_done_fn )Ifx_Lwip_init, (void*)&ethAddr);

    /*Initialize the AUDIO Tasks*/
    AudioTasksCreate();
    /*Initialize the UART Task*/
    UartTaskCreate();

    xTaskCreate(OsTask_IdleLed, "LED task", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY, NULL);

    /* Start the tasks running. */
    vTaskStartScheduler();
    
    while(1)
    {
    }
}
