/**
 * \file Ifx_Lwip.c
 * \brief
 *
 * \version V0.1
 * \copyright Copyright (c) 2015 Infineon Technologies AG. All rights reserved.
 *
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 */

/******************************************************************************/
/*----------------------------------Includes----------------------------------*/
/******************************************************************************/
#include <Cpu/Std/Ifx_Types.h>
#include <Cpu/Std/IfxCpu.h>
#include "IfxGeth_Eth.h"
#include "Ifx_Lwip.h"
#include "Ifx_netif.h"
#include "lwipopts.h"
#include "Ifx_Netif.h"
#include "tcpip.h"


#include <string.h>
#include <stdarg.h>
#include "Ifx_Console.h"
#include "IsrTaskPrios.h"

#include "FreeRTOS.h"
#include "task.h"
#include "portmacro.h"

#include "IfxGeth_Phy_Dp83825i.h"
/******************************************************************************/
/*-----------------------------------Macros-----------------------------------*/
/******************************************************************************/
#define IFX_LWIP_TIMER_TICK_MS      (1U)    // number of timer ticks per millisecond

#define IFX_LWIP_ARP_PERIOD         (ARP_TMR_INTERVAL / IFX_LWIP_TIMER_TICK_MS)
#define IFX_LWIP_TCP_FAST_PERIOD    (TCP_FAST_INTERVAL / IFX_LWIP_TIMER_TICK_MS)
#define IFX_LWIP_TCP_SLOW_PERIOD    (TCP_SLOW_INTERVAL / IFX_LWIP_TIMER_TICK_MS)
#define IFX_LWIP_DHCP_COARSE_PERIOD (DHCP_COARSE_TIMER_MSECS / IFX_LWIP_TIMER_TICK_MS)
#define IFX_LWIP_DHCP_FINE_PERIOD   (DHCP_FINE_TIMER_MSECS / IFX_LWIP_TIMER_TICK_MS)
#define IFX_LWIP_LINK_PERIOD        (100U / IFX_LWIP_TIMER_TICK_MS) /* 100 ms */

#define IFX_LWIP_FLAG_ARP           (1U << 1)
#define IFX_LWIP_FLAG_TCP_FAST      (1U << 2)
#define IFX_LWIP_FLAG_TCP_SLOW      (1U << 3)
#define IFX_LWIP_FLAG_LINK          (1U << 4)
#define IFX_LWIP_FLAG_DHCP_COARSE   (1U << 5)
#define IFX_LWIP_FLAG_DHCP_FINE     (1U << 6)

/******************************************************************************/
/*--------------------------------Enumerations--------------------------------*/
/******************************************************************************/

/******************************************************************************/
/*-----------------------------Data Structures--------------------------------*/
/******************************************************************************/
#define CPU_WHICH_SERVICE_ETHERNET 0
/******************************************************************************/
/*------------------------------Global variables------------------------------*/
/******************************************************************************/
#if CPU_WHICH_SERVICE_ETHERNET == 0
    #if defined(__GNUC__)
    #pragma section ".text_cpu0" ax
    #pragma section ".bss_cpu0" awc0
    #endif
    #if defined(__TASKING__)
    #pragma section code    "text_cpu0"
    #pragma section farbss  "bss_cpu0"
    #pragma section fardata "data_cpu0"
    #endif
    #if defined(__DCC__)
    #pragma section CODE ".text_cpu0"
    #pragma section DATA ".data_cpu0" ".bss_cpu0" far-absolute RW
    #endif
    #if defined(__ghs__)
    #pragma ghs section text=".text_cpu0"
    #pragma ghs section bss= ".bss_cpu0"
    #pragma ghs section data=".data_cpu0"
    #endif
#elif ((CPU_WHICH_SERVICE_ETHERNET == 1) && (CPU_WHICH_SERVICE_ETHERNET < IFXCPU_NUM_MODULES))
    #if defined(__GNUC__)
    #pragma section ".text_cpu1" ax
    #pragma section ".bss_cpu1" awc1
    #endif
    #if defined(__TASKING__)
    #pragma section code    "text_cpu1"
    #pragma section farbss  "bss_cpu1"
    #pragma section fardata "data_cpu1"
    #endif
    #if defined(__DCC__)
    #pragma section CODE ".text_cpu1"
    #pragma section DATA ".data_cpu1" ".bss_cpu1" far-absolute RW
    #endif
    #if defined(__ghs__)
    #pragma ghs section text=".text_cpu1"
    #pragma ghs section bss= ".bss_cpu1"
    #pragma ghs section data=".data_cpu1"
    #endif
#elif ((CPU_WHICH_SERVICE_ETHERNET == 2) && (CPU_WHICH_SERVICE_ETHERNET < IFXCPU_NUM_MODULES))
    #if defined(__GNUC__)
    #pragma section ".text_cpu2" ax
    #pragma section ".bss_cpu2" awc2
    #endif
    #if defined(__TASKING__)
    #pragma section code    "text_cpu2"
    #pragma section farbss  "bss_cpu2"
    #pragma section fardata "data_cpu2"
    #endif
    #if defined(__DCC__)
    #pragma section CODE ".text_cpu2"
    #pragma section DATA ".data_cpu2" ".bss_cpu2" far-absolute RW
    #endif
    #if defined(__ghs__)
    #pragma ghs section text=".text_cpu2"
    #pragma ghs section bss= ".bss_cpu2"
    #pragma ghs section data=".data_cpu2"
    #endif
#elif ((CPU_WHICH_SERVICE_ETHERNET == 3) && (CPU_WHICH_SERVICE_ETHERNET < IFXCPU_NUM_MODULES))
	#if defined(__GNUC__)
    #pragma section ".text_cpu3" ax
	#pragma section ".bss_cpu3" awc3
	#endif
	#if defined(__TASKING__)
    #pragma section code    "text_cpu3"
    #pragma section farbss  "bss_cpu3"
    #pragma section fardata "data_cpu3"
	#endif
	#if defined(__DCC__)
    #pragma section CODE ".text_cpu3"
	#pragma section DATA ".data_cpu3" ".bss_cpu3" far-absolute RW
	#endif
    #if defined(__ghs__)
    #pragma ghs section text=".text_cpu3"
    #pragma ghs section bss= ".bss_cpu3"
    #pragma ghs section data=".data_cpu3"
    #endif
#elif ((CPU_WHICH_SERVICE_ETHERNET == 4) && (CPU_WHICH_SERVICE_ETHERNET < IFXCPU_NUM_MODULES))
	#if defined(__GNUC__)
    #pragma section ".text_cpu4" ax
	#pragma section ".bss_cpu4" awc4
	#endif
	#if defined(__TASKING__)
    #pragma section code    "text_cpu4"
    #pragma section farbss  "bss_cpu4"
    #pragma section fardata "data_cpu4"
	#endif
	#if defined(__DCC__)
    #pragma section CODE ".text_cpu4"
	#pragma section DATA ".data_cpu4" ".bss_cpu4" far-absolute RW
	#endif
    #if defined(__ghs__)
    #pragma ghs section text=".text_cpu4"
    #pragma ghs section bss= ".bss_cpu4"
    #pragma ghs section data=".data_cpu4"
    #endif
#elif ((CPU_WHICH_SERVICE_ETHERNET == 5) && (CPU_WHICH_SERVICE_ETHERNET < IFXCPU_NUM_MODULES))
	#if defined(__GNUC__)
    #pragma section ".text_cpu5" ax
	#pragma section ".bss_cpu5" awc5
	#endif
	#if defined(__TASKING__)
    #pragma section code    "text_cpu5"
    #pragma section farbss  "bss_cpu5"
    #pragma section fardata "data_cpu5"
	#endif
	#if defined(__DCC__)
    #pragma section CODE ".text_cpu5"
	#pragma section DATA ".data_cpu5" ".bss_cpu5" far-absolute RW
	#endif
    #if defined(__ghs__)
    #pragma ghs section text=".text_cpu5"
    #pragma ghs section bss= ".bss_cpu5"
    #pragma ghs section data=".data_cpu5"
    #endif
#else
#error "Set CPU_WHICH_SERVICE_ETHERNET to a valid value!"
#endif

/* The time to block waiting for input. */
#define emacBLOCK_TIME_WAITING_FOR_INPUT	( ( portTickType ) 100 )

Ifx_Lwip    g_Lwip;
IfxGeth_Eth g_IfxGeth;
/* The semaphore used to signal the arrival of new data to the interface
task. */

uint32 isrTxCount=0;
uint32 isrRxCount=0;
uint8 channel0TxBuffer1[IFXGETH_MAX_TX_DESCRIPTORS][IFXGETH_MAX_TX_BUFFER_SIZE];
uint8 channel0RxBuffer1[IFXGETH_MAX_RX_DESCRIPTORS][IFXGETH_MAX_RX_BUFFER_SIZE];


/******************************************************************************/
/*------------------------Private Variables/Constants-------------------------*/
/******************************************************************************/
#if defined(__GNUC__)
    #pragma section // end bss section
#endif

/******************************************************************************/
/*-------------------------Function Prototypes--------------------------------*/
/******************************************************************************/
/**
 * This interrupt is raised by the ethernet tx. The initialization is done by IfxGeth_Eth_init().
 *
 * \isrProvider \ref ISR_PROVIDER_ETH
 * \isrPriority \ref ISR_PRIORITY_GETH_TX
 *
 */

IFX_INTERRUPT(ISR_Geth_Tx, CPU_WHICH_SERVICE_ETHERNET, Isr_Eth_Tx);

void ISR_Geth_Tx()
{

    isrTxCount++;
}

/**
 * This interrupt is raised by the ethernet rx. The initialization is done by IfxGeth_Eth_init().
 *
 * \isrProvider \ref ISR_PROVIDER_ETH
 * \isrPriority \ref ISR_PRIORITY_GETH_RX
 *
 */
IFX_INTERRUPT(ISR_Geth_Rx, CPU_WHICH_SERVICE_ETHERNET, Isr_Eth_Rx);

void ISR_Geth_Rx()
{

	isrRxCount++;
	portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;
	vTaskNotifyGiveFromISR( g_Lwip.EthRxTask, &xHigherPriorityTaskWoken );

    /* If a task was woken by either a frame being received then we may need to
	switch to another task.  If the unblocked task was of higher priority then
	the interrupted task it will then execute immediately that the ISR
	completes. */

	portYIELD_FROM_ISR(xHigherPriorityTaskWoken);



}

void Ifx_Lwip_WaitForInput( void )
{
	/* Just wait until we are signled from an ISR that data is available, or
	we simply time out. */
    ulTaskNotifyTake(pdFALSE, emacBLOCK_TIME_WAITING_FOR_INPUT);

}

/******************************************************************************/
/*-------------------------Function Implementations---------------------------*/
/******************************************************************************/

void Ifx_GethLinkStatus(void *pvParameter)
{

	Ifx_GETH_MAC_PHYIF_CONTROL_STATUS ctrl_status;

	for(;;)
	{
		ctrl_status.U = IfxGeth_Eth_Phy_Dp83825i_link_status();
		if (ctrl_status.B.LNKSTS == 0){
			netif_set_link_down(&g_Lwip.netif);
			LWIP_DEBUGF(IFX_LWIP_LINK_DEBUG, ("Ifx_GethLinkStatus link is down!\n"));
		}else {
			IfxGeth_Eth *ethernetif = g_Lwip.netif.state;
			// we set the correct duplexMode
			if (ctrl_status.B.LNKMOD == 1)
				IfxGeth_mac_setDuplexMode(ethernetif->gethSFR, IfxGeth_DuplexMode_fullDuplex);
			else
				IfxGeth_mac_setDuplexMode(ethernetif->gethSFR, IfxGeth_DuplexMode_halfDuplex);
			// we set the correct speed
			if (ctrl_status.B.LNKSPEED == 0)
				// 10MBit speed
				IfxGeth_mac_setLineSpeed(ethernetif->gethSFR, IfxGeth_LineSpeed_10Mbps);
			else
				if (ctrl_status.B.LNKSPEED == 1)
					// 100MBit speed
					IfxGeth_mac_setLineSpeed(ethernetif->gethSFR, IfxGeth_LineSpeed_100Mbps);
				else
					// 1000MBit speed
					IfxGeth_mac_setLineSpeed(ethernetif->gethSFR, IfxGeth_LineSpeed_1000Mbps);
			netif_set_link_up(&g_Lwip.netif);
			LWIP_DEBUGF(IFX_LWIP_LINK_DEBUG, ("Ifx_GethLinkStatus link is up!\n"));
		}
		vTaskDelay(1000);  // Check every second the Link (Cable Plugged or not)
	}
}





//________________________________________________________________________________________
// INITIALIZATION FUNCTION

/** \brief LWIP initialization function
 *
 * The followings are executed: */
void Ifx_Lwip_init(eth_addr_t *ethAddr)
{
    ip_addr_t default_ipaddr, default_netmask, default_gw;
    IP4_ADDR(&default_gw, 192,168,2,1);
    IP4_ADDR(&default_ipaddr, 192,168,2,218);
    IP4_ADDR(&default_netmask, 255,255,255,0);

    LWIP_DEBUGF(IFX_LWIP_DEBUG, ("Ifx_Lwip_init start!\n"));

    /** - initialise LWIP (lwip_init()) */
    //lwip_init();


    /** - initialise and add a \ref netif */
    g_Lwip.eth_addr = *ethAddr;
    netif_add(&g_Lwip.netif, &default_ipaddr, &default_netmask, &default_gw,
        (void *)0, ifx_netif_init, tcpip_input);




    // Create a Task for checking the Link
    xTaskCreate(Ifx_GethLinkStatus,"Eth Link Stat",configMINIMAL_STACK_SIZE,NULL,TaskPrio_EthLinkStatus,NULL);
    // Task is woken up by an Rx Ethernet Event using Binary Semaphore
    xTaskCreate(ifx_netif_input,"Eth RX",512,&g_Lwip.netif,TaskPrio_EthMacRX,&g_Lwip.EthRxTask);

    netif_set_default(&g_Lwip.netif);
    netif_set_up(&g_Lwip.netif);

#if LWIP_NETIF_HOSTNAME
    g_Lwip.netif.hostname = BOARDNAME;
#endif

//    /** - assign \ref dhcp to \ref netif */
//    dhcp_set_struct(&g_Lwip.netif, &g_Lwip.dhcp);
//    /* we start the dhcp always here also when we don't have a link here */
//    dhcp_start(&g_Lwip.netif);
//    //TODO IFX_Lwip.c (451): May stay in here until we have a link and DHCP Address
//    for(int i=0; i < 10 && g_Lwip.dhcp.state != 10; i++) {
//    	LWIP_DEBUGF(IFX_LWIP_DEBUG, (" Current DHCP State : %d\n",g_Lwip.dhcp.state));
//        // Wait a second
//        vTaskDelay(1000);
//     }



        // Make it active ...
    LWIP_DEBUGF(IFX_LWIP_DEBUG, ("Interface is up : %d\n",g_Lwip.dhcp.state));

    LWIP_DEBUGF(IFX_LWIP_DEBUG, ("IP %d\n",g_Lwip.netif.ip_addr));

    LWIP_DEBUGF(IFX_LWIP_DEBUG, ("NM %d\n",g_Lwip.netif.netmask));

    LWIP_DEBUGF(IFX_LWIP_DEBUG, ("GW %d\n",g_Lwip.netif.gw));

    LWIP_DEBUGF(IFX_LWIP_DEBUG, ("Ifx_Lwip_init end!\n"));
}




//________________________________________________________________________________________
// DEBUGGING FUNCTIONS
#include "configuration.h"
#include <stdio.h>
#include <string.h>

#define MAXCHARS 256

#ifdef LWIP_PRINT_VIA_DISPLAY
#include "conio_tft.h"
#endif
s8_t Ifx_Lwip_printf(const char *format, ...)
{
#ifdef __LWIP_DEBUG__
    char    str[MAXCHARS + 4];
    s8_t    result = ERR_CONN;

    va_list args;
    va_start(args, format);
    vsnprintf(str, MAXCHARS, format, args);
    va_end(args);
    {
        sendUARTMessage(str, strlen(str));
#ifdef LWIP_PRINT_VIA_DISPLAY
    	conio_ascii_cputs (DISPLAYSTDOUT1, (uint8 *)&str[0]);
#endif
#ifdef LWIP_PRINT_VIA_CONSOLE
        Ifx_Console_print(&str[0]);
        if (str[strlen(str)-1] == '\n') Ifx_Console_print("\r");
#endif
    }
#endif
    return result;
}

#if defined(__GNUC__)
#pragma section // end text section
#endif
#if defined(__TASKING__)
#pragma section code restore
#pragma section fardata restore
#pragma section farbss restore
#endif
#if defined(__DCC__)
#pragma section CODE
#pragma section DATA RW
#endif
#if defined(__ghs__)
#pragma ghs section text=default
#pragma ghs section data=default
#pragma ghs section bss=default
#endif

