#TC375 Lite Kit FreeRTOS + Audio Shield Board + Imagimob Studio
This Example is based on TC375 Lite Kit with the combination of Automotive MEMS Microphones on the Audio Shield Board to demonstrate AI based Application for Siren Detection.<br>
- This project is based on an Infineon [Aurix_AudioAppKit](https://github.com/Infineon/Aurix_AudioAppKit/) project .<br>
- This project is extended with Ethernet and LwIp to stream PCM data via UDP. <br>
- This project is extended with additional task to process the audio data in AI model "Siren Detection".<br>
- The AI model for siren detection is based on the Imagimob Studio starter project "Siren Detection" and is integrated in this project with permission  of Imagimob.

##Device
<b>
- 1 x Aurix™ 2G TC375TP - AA Step <br>
- 2 x MEMS Microphones IM67D120A <b>


##Board
AURIX™ TC375 Lite Kit Board<br>
<img src="./doc/LiteKit.png" width="1500" /> 
<br>
KITA2G Audio Shield Board<br>
<img src="./doc/AudioShield.png" width="1500" />


##Scope of Work
The Scope of this Project is to showcase the following features:
- receiving PDM stream from the Microphones (IM67D120A) via QSPI
- process the PDM stream and convert the PDM to PCM stream
- process the PCM stream in the AI model to recognize siren sound.
- print the results of recognition via serial interface (UART) to a terminal
- additionally the 16kHZ PCM mono stream is streamed via UDP to a host PC

##Third-Party Packages
- This software package includes the FreeRTOS V10.4.6
- This software package includes the lwIP - A Lightweight TCP/IP stack(2.1.2)
- This software package includes the OpenPDMFilter v1.0.0

##Hardware setup
<img title="Hardware Setup" alt="Hardware Setup" src="doc/TC375_MIC_LB.png" width="1500">

## Compiling
The example projects can be compiled and debugged with the [Aurix™ Development Studio](https://www.infineon.com/cms/en/product/promopages/aurix-development-studio/) (v1.9.12 upwards).<br>
<br> 
<img src="./doc/AURIX_Development_Studio.png" width="1500" />

##Implementation
The Project contains following folder with the following content:
- Configurations folder contains user configurations for FreeRTOS, BootModeHeader and LwIp.
- Libraries  folder contains all Driver and Middleware Software used in the project:<br>
    - *AudioShield folder*: contains the configuration for the KITA2G Audio Shield Board and the OpenPDM library.<br>
    - *Ethernet folder*: contains the LwIp Ethernet Stack and the Ethernet Transceiver  Driver.<br>
    - *FreeRTOS folder*: contains the FreeRTOS and abstraction layer for AURIX™<br>
    - *iLLD folder*: contains the Infineon Low Level Driver for all the AURIX™ modules, peripherals.<br> 
    - *Infra folder*: contains the Startup Code and special function register definition.<br>
    - *SirenDetection folder*: contains AI Model generated by Imagimob Studio. To integrate a new 
    Imagimob Studio Model (siren detection) the model.c and model.h files has to be replaced in this folder<br>
    - *UART folder*: contains the serial communication abstraction.<br> 
- TriCore Debug folder contains the build artifacts (*.map, .elf, .hex) after a build of the project.
- The Top folder contains the FreeRTOS Task implementation and the start code for each CPU. <br>

The complete application is running on Cpu0. The entry point for the application is the *core0_main* function. 
In this function the LwIp, Interrupts get initialized also the Tasks for AUDIO and Uart get created. 
The last step in this Function is the start of the FreeRTOS task scheduling. <br> 
Following AUDIO Tasks are running in the application:
- OsTask_ExternalAudioControl: reads the ADC values from an Potentiometer to setup the volume for the AUDIO output.
- OsTask_AudioProcess: read the PDM values (transferred via DMA from the SPI). Separate the 2 channel PDM data and transform each of the PDM to PCM channels. 
One of the channels is written into a circular buffer to be processed in the Imagimob Studio Model. 
Additionally the 2 PCM channels get merged to an PCM Stream and output via AUDIO Amplifier.
- OsTask_DataProcessing: This Task is getting the PCM 16 kHZ mono channel data from circular buffer and que this data to Siren Detection.
<img src="./doc/Processing_Data.png" width="1500" />  
- OsTask_UART: This Task prints the result buffer to Serial Interface. 

##Run and Test
The software can be flashed with Aurix™ Development Studio see the chapter Compiling. The functionality of siren detection can be tested with following 
[Test Vidoe](https://www.youtube.com/watch?v=3TiQB7_YfZw) with different siren sounds. To see the results of the siren detection an terminal window need to be opened. 
Therefore Aurix™ Development Studio Terminal or another Terminal software like [PuTTY](https://putty.org/) can be used. <br>
<img src="./doc/Terminal_connection.png" width="1000" /> <br>
The Results can be observed in the Terminal Window. <br>
<img src="./doc/Results.png" width="1000" /> <br>
For the UDP PCM raw data stream the IP Address on the HOST side has to be configured as "192.168.2.10" with the MAC "255.255.255.0".
In [Wireshark](https://www.wireshark.org/) the UDP steram looks like folowing<br>
<img src="./doc/UDP_Stream.png" width="1000" /> <br>


##References
[Imagimob](https://www.imagimob.com/) <br>
[Aurix_AudioAppKit](https://github.com/Infineon/Aurix_AudioAppKit/)<br>
[Aurix™ Development Studio](https://www.infineon.com/cms/en/product/promopages/aurix-development-studio/) <br>
[Siren Test Video](https://www.youtube.com/watch?v=3TiQB7_YfZw)<br>
[Wireshark](https://www.wireshark.org/)<br>
[PuTTY](https://putty.org/)


### Infineon   
<img src="./doc/IFX_LOGO_600.gif">


